@@ -1,12 +1,13 @@
 import json
 import operator
 import re
+from functools import lru_cache
 
 from django.apps import apps
 from django.contrib.admin.utils import unquote
 from django.contrib.messages import get_messages
 from django.core import signing
-from django.core.exceptions import PermissionDenied, ValidationError, FieldError
+from django.core.exceptions import FieldError, PermissionDenied, ValidationError
 from django.db import transaction
 from django.forms.fields import CharField
 from django.http import (
@@ -49,7 +50,7 @@ from cms.utils.urlutils import admin_reverse
 from . import settings
 from .forms import ActionTokenValidationForm, RenderPluginForm, TextForm
 from .html import render_dynamic_attributes
-from .models import Text
+from .models import Text, _MAX_RTE_LENGTH
 from .utils import (
     OBJ_ADMIN_WITH_CONTENT_RE_PATTERN,
     _plugin_tags_to_html,
@@ -249,6 +250,7 @@ class TextPlugin(CMSPluginBase):
                 action_token=action_token,
                 revert_on_cancel=settings.TEXT_CHILDREN_ENABLED and rte_config.child_plugin_support,
                 body_css_classes=self._get_body_css_classes_from_parent_plugins(plugin),
+                add_admin_css=True,
             )
         else:
             widget = TextEditorWidget(
@@ -315,6 +317,7 @@ class TextPlugin(CMSPluginBase):
             rendered_text = plugin_tags_to_admin_html(
                 text=instance.body,
                 context=context,
+                child_plugin_instances=instance.child_plugin_instances,
             )
         else:
             rendered_text = None
@@ -535,7 +538,7 @@ class TextPlugin(CMSPluginBase):
             # django CMS 3
             qs = list(
                 PageContent.objects.filter(language=language, title__icontains=search).order_by("page__node__path")
-                )
+            )
             for page_content in qs:
                 # Patch the missing get_absolute_url method
                 page_content.get_absolute_url = lambda: page_content.page.get_absolute_url()
@@ -575,9 +578,9 @@ class TextPlugin(CMSPluginBase):
         )
 
     @classmethod
-    def get_child_plugin_candidates(cls, slot, page):
+    @lru_cache
+    def get_child_plugin_candidates(cls, *args, **kwargs):
         """
-
         This method is a class method that returns a list of child plugin candidates for a given slot and page.
 
         Parameters:
@@ -587,10 +590,7 @@ class TextPlugin(CMSPluginBase):
         Returns:
         - A list of text-enabled plugins that can be used as child plugins for the given slot and page.
         """
-        text_enabled_plugins = plugin_pool.get_text_enabled_plugins(
-            placeholder=slot,
-            page=page,
-        )
+        text_enabled_plugins = plugin_pool.get_text_enabled_plugins(*args, **kwargs)  # Just pass on arguments
         # Filter out plugins that are not in the whitelist if given
         if settings.TEXT_CHILDREN_WHITELIST is not None:
             text_enabled_plugins = [
@@ -603,7 +603,8 @@ class TextPlugin(CMSPluginBase):
             ]
         return text_enabled_plugins
 
-    def render_plugin_icon(self, plugin):
+    @staticmethod
+    def render_plugin_icon(plugin: CMSPluginBase):
         icon = getattr(plugin, "text_icon", None)
         if icon is None:
             return
@@ -616,9 +617,10 @@ class TextPlugin(CMSPluginBase):
         if not plugin or not TEXT_CHILDREN_ENABLED or not rte_config.child_plugin_support:
             return []
         get_plugin = plugin_pool.get_plugin
+        page = self.placeholder.page if hasattr(self.placeholder, "page") else None
         child_plugin_types = self.get_child_classes(
             slot=plugin.placeholder.slot,
-            page=self.page,
+            page=page,
         )
         child_plugins = (get_plugin(name) for name in child_plugin_types)
         template = getattr(self.page, "template", None)
@@ -679,7 +681,9 @@ class TextPlugin(CMSPluginBase):
 
             context.update(
                 {
-                    "body": plugin_tags_to_admin_html(body, context),
+                    "body": plugin_tags_to_admin_html(
+                        body, context, child_plugin_instances=instance.child_plugin_instances
+                    ),
                     "placeholder": placeholder,
                     "object": instance,
                     "editor_settings": editor_settings,
@@ -692,7 +696,9 @@ class TextPlugin(CMSPluginBase):
             body = render_dynamic_attributes(instance.body, admin_objects=False, remove_attr=True)
             context.update(
                 {
-                    "body": plugin_tags_to_user_html(body, context),
+                    "body": plugin_tags_to_user_html(
+                        body, context, child_plugin_instances=instance.child_plugin_instances
+                    ),
                     "placeholder": placeholder,
                     "object": instance,
                 }
@@ -712,7 +718,7 @@ class TextPlugin(CMSPluginBase):
                 # subclassing cms_plugin_instance (one to one relation)
                 value = getattr(self.cms_plugin_instance, field.name)
                 setattr(obj, field.name, value)
-        obj.rte = rte_config.name
+        obj.rte = rte_config.name[:_MAX_RTE_LENGTH]
         super().save_model(request, obj, form, change)
         # This must come after calling save
         # If `clean_plugins()` deletes child plugins, django-treebeard will call
